
function FGTileColor(name, style)
{
   return NamedColor(name,null).className("FGTileColor")
}

Colors = {} 

// MARK: Tile Colors

function BuildTileColors(colors)
{
    for  (var v in colors)
    {
        // Create the base tile color
        if (Colors[colors[v]] == null)
        {
            Colors[colors[v]] = MainThemeColor(FGTileColor(`${colors[v]}.lightMode`), FGTileColor(`${colors[v]}.darkMode`))
        }
        // Add various styles
        for (var c = 0; c < 9; c++)
        {
            let styledColor = MainThemeColor(FGTileColor(`${colors[v]}.lightMode.${c}`), FGTileColor(`${colors[v]}.darkMode.${c}`)).showDisabled(Colors.tertiaryContent);
            Colors[colors[v]] = Colors[colors[v]].addStyle(`${c}`, styledColor.showDisabled(Colors.secondaryContent.withAlpha(0.5))); 
        } 
    }
}      

Colors.white                   = colorRGBA(255, 255, 255, 100);
Colors.pitchBlack              = colorRGBA(0, 0, 0, 100);
Colors.black                   = colorRGBA(55, 55, 55, 100);
Colors.invisible               = MainThemeColor(colorRGBA(255, 255, 255, 0), colorRGBA(255, 255, 255, 0)); 

   
Colors.fontColor               = MainThemeColor(colorGray(51,100), colorGray(251,100))  

Colors.mainBackground          = MainThemeColor(colorGray(251,100), colorGray(33,100)) 
Colors.secondaryBackground     = MainThemeColor(colorGray(233, 100), colorGray(51, 100)) 
Colors.secondaryBackgroundMix  = MainThemeColor(colorGray(214, 100), colorGray(70, 100)).withAlpha(0.5)
Colors.tileBackground          = MainThemeColor(colorGray(255, 100), colorGray(51, 100)).addStyle("disabled", MainThemeColor(colorGray(255, 100), colorGray(55, 100)))
Colors.tertiaryContent         = MainThemeColor(colorGray(209,100), colorGray(136,100))
Colors.secondaryContent        = MainThemeColor(colorGray(115,100), colorGray(115,100)).showDisabled(Colors.tertiaryContent); 
Colors.toolbarButtonDark       = MainThemeColor(Colors.black, Colors.white).showDisabled(MainThemeColor(Colors.black, Colors.white).withAlpha(0.4));  
Colors.toolbarButton           = Colors.secondaryContent.showDisabled(Colors.secondaryContent.withAlpha(0.4));  

Colors.primary          = MainThemeColor(colorRGBA(29, 35, 35, 100),    colorRGBA(255, 255, 255, 100)); 
Colors.tertiary         = MainThemeColor(colorRGBA(180, 180, 180, 100), colorRGBA(170, 170, 170, 100));   
Colors.key              = MainThemeColor(colorRGBA(114,52,220,100),     colorRGBA(170,133,234,100));
Colors.warning          = MainThemeColor(colorRGBA(246,8,0,100),        colorRGBA(246,8,0,100));
Colors.yellow           = MainThemeColor(colorRGBA(245,151,0,100),      colorRGBA(255,182,0,100));  // Same as tile yellow
Colors.primaryContent   = MainThemeColor(Colors.black, Colors.white).showDisabled(Colors.tertiaryContent);
Colors.debug            = Colors.invisible.addStyle("debug", Colors.warning);

Colors.shadowColor       = MainThemeColor(Colors.black.withAlpha(0.2), Colors.invisible);

// Build tile colors
BuildTileColors(["tileTintPale", "tileTint"]); 

Colors.tileTintLight    = MainThemeColor(Colors.tileTint.saturation(0.4).brightness(1.1).addStyle("8", Colors.secondaryContent), Colors.tileTint.saturation(0.6).brightness(0.5));
Colors.tileTintPrimary  = MainThemeColor(Colors.tileTint.lightMode['0'], Colors.tileTint.darkMode['0']);

Colors.controlFill      = MainThemeColor(colorRGBA(255,255,255,100), Colors.secondaryBackground)
                                .addStyle("on", Colors.tileTint.saturation(0.75))
                                .addStyle("pressed", Colors.tileTint.brightness(1.01));
                                
Colors.baseControlContent = MainThemeColor(Colors.black.addStyle("on", Colors.white.addStyle("8", Colors.black)), 
                                           Colors.white.addStyle("on", Colors.white.addStyle("8", Colors.black)))

Colors.controlContent   =  Colors.baseControlContent 
                                 .addStyle("mixed", Colors.white)
                                 .addStyle("disabled", Colors.baseControlContent.addStyle("locked", Colors.white))  
                                 .addStyle("pressed", Colors.white.brightness(0.9).addStyle("8", Colors.black.brightness(1.1)))
                                 .addStyle("8", MainThemeColor(Colors.white.addStyle("on", Colors.white.showPressed()),
                                                               Colors.black.addStyle("on", Colors.black.showPressed())
                                                               .addStyle("key", Colors.black))
                                          ); 

Colors.contrastBorder         = MainThemeColor(Colors.black, Colors.white.withAlpha(0.5));
Colors.contrast               = Colors.invisible.addStyle("increasedContrast", Colors.contrastBorder);

Colors.divider                = Colors.secondaryBackground.addStyle("increasedContrast", Colors.contrastBorder)

// Set list colors                                         
Colors.setListText = MainThemeColor(Colors.black, Colors.white.brightness(0.8))    
        .addStyle("pressed", MainThemeColor(Colors.white, Colors.black))
        .addStyle("selected", MainThemeColor(Colors.black, Colors.secondaryBackground.darkMode))
        .addStyle("focused", MainThemeColor(Colors.white, Colors.white))
  
Colors.setListTextSecondary = Colors.secondaryContent
                                 .addStyle("selected", MainThemeColor(Colors.black, Colors.secondaryBackground.darkMode))
                                 .addStyle("pressed", MainThemeColor(Colors.white, Colors.black)) 
                                 .addStyle("focused", MainThemeColor(Colors.white, Colors.white))

Colors.editorPlayhead = MainThemeColor(colorRGBA(255, 0, 0, 100)) 


Colors.highlightActive = Colors.tileTintLight;

Colors.knobColor = Colors.white.addStyle("pressed", Colors.white.brightness(0.9));

Colors.tilePillBack = 
{
	nsClass: "NSColor",
	"lightMode" : Colors.tileTintPale.showPressed(),
	"darkMode" : Colors.tileTintPale.showPressed(),
	"playing" : Colors.tileTint.showPressed(),
} 

Colors.tilePillFont = {
	nsClass: "NSColor",
	"lightMode" : Colors.tileTint.showPressed(),
	"darkMode" : Colors.tileTint.showPressed(),
	"playing" : MainThemeColor(Colors.white.addStyles(["0","1","2","3","4","5","6","7","8"], Colors.white),
							   Colors.tileBackground.darkMode.addStyles(["0","1","2","3","4","5","6","7","8"], Colors.tileBackground.darkMode)),
}

Colors.tileWaveForeColor = {
	nsClass: "NSColor",
	"lightMode" : Colors.tileBackground,
	"darkMode" : Colors.tileBackground,
	"playing" :	Colors.tileTintPale,
    "preview" :	Colors.tileTintPale,
}

// Color that inverses when dark mode is enabled
Colors.tileTintContent = Colors.white.addStyle("8", MainThemeColor(Colors.white, Colors.black).addStyle("locked", Colors.fontColor.inverted())).showPressed();

markup.push({

	"Colors": {   
        "muted": "rgba:255,60,60,100",
        "mainBackground.lightMode" : "gray:251,100",
        "mainBackground.darkMode" : "gray:33,100",
        "secondaryBackground.lightMode": "gray:233,100",
        "secondaryBackground.darkMode": "gray:51,100",
        "invisible": "gray:0,0", 
        "trialFill": {
            // these are 10 colors, mostly repeated to make 20 total.
            // the last color is a bigger jump to red and is not duped.
            "0": "rgba:172, 131, 242, 100",
            "1": "rgba:172, 131, 242, 100",
            "2": "rgba:172, 131, 242, 100",
            "3": "rgba:172, 131, 242, 100",
            "4": "rgba:174, 127, 232, 100",
            "5": "rgba:174, 127, 232, 100",
            "6": "rgba:176, 124, 223, 100",
            "7": "rgba:176, 124, 223, 100",
            "8": "rgba:179, 121, 213, 100",
            "9": "rgba:179, 121, 213, 100",
            "10": "rgba:181, 117, 204, 100",
            "11": "rgba:181, 117, 204, 100",
            "12": "rgba:183, 114, 194, 100",
            "13": "rgba:183, 114, 194, 100",
            "14": "rgba:186, 111, 184, 100",
            "15": "rgba:186, 111, 184, 100",
            "16": "rgba:188, 107, 175, 100",
            "17": "rgba:188, 107, 175, 100",
            "18": "rgba:196, 97, 147, 100",
            "19": "rgba:196, 97, 147, 100",
            "20": "rgba:220, 65, 52, 100",
        },
	},

});
