function uuidv4() 
{ 
    if (!this.counter)
    {
        this.counter = 0;
    } 
    this.counter += 1;
    return `_${this.counter}`;
}

function print(val)
{
    console.log(val);
}

JSON.safeStringify = (obj, indent = 2) => {
    let cache = [];
    const retVal = JSON.stringify(
      obj,
      (key, value) =>
        typeof value === "object" && value !== null
          ? cache.includes(value)
            ? undefined // Duplicate reference found, discard key
            : cache.push(value) && value // Store value in our collection
          : value,
      indent
    );
    cache = null;
    return retVal;
};

class Base
{   
    constructor()
    { 
        this.addedStyles = []; 
    }

    copy()
    { 
        let child = this; 
        var cpy = Object.assign(Object.create(Object.getPrototypeOf(child)), child);  
        return cpy;
    }
   
    setValue(value, key, recurse = false)
    { 
        var cpy = this.copy();
        if (recurse)
        {
            cpy = this.forAllChildren((child) => { 
                if (child[key] != null)
                {
                    return child; // Don't overwrite
                }
                return child.setValue(value, key) 
            }); 
        } 
        cpy[key] = value;
        return cpy;
    }
    
    setValues(values)
    { 
        var cpy = this.copy();  
        return Object.assign(cpy, values);
    }
    
    dump()
    {
        console.log(JSON.safeStringify(this));
        return this;
    } 
    
    shadow(color=Colors.shadowColor,x=0,y=-1,radius=2)
    { 
        if (color == null)
        {
            return this.setValue(null, "gxShadowColor");
        }
        return this.setValue(Shadow(color,x,y,radius), "gxShadow");
    }

    backColor(color)
    {
        return this.setValue(color, "gxBackColor");
    }
    
    className(cls)
    {
        if (cls == null)
        {
            return this;
        }
        return this.setValue(cls, "nsClass");
    }
    
    keyPath(keyPath)
    {
        var cpy = this.forAllChildren((child) => { 
            if (child.gxValueKeyPath != null)
            {
                return child; // Don't overwrite
            }
            return child.keyPath(keyPath) 
        });   
        if (keyPath != null)
        {
            cpy["gxValueKeyPath"] = keyPath
        } 
        return cpy;
    }
    
    children()
    {
        if (this["gxRootLayer"] != null)
        {
            if (this["gxRootLayer"]["gxSublayers"] != null)
            {
                return this["gxRootLayer"]["gxSublayers"];
            } 
        }
        if (this["gxSublayers"] != null)
        {
            return this["gxSublayers"];
        }
        return this.gxChildViews;
    }

    forAllChildren(action)
    {
        var cpy = this.copy(); 
        var children = cpy.children(); 
        for (var v in children)
        {  
            let child = children[v]
            // print(child.constructor.name);
            // Check if the child cunstructor name contains "Base"
            if (child instanceof BaseView)
            {
                children[v] = action(child);
            } 
        }
        return cpy;
    }

    keyEquivalent(key)
    {
        var cpy = this.forAllChildren((child) => { 
            if (child.gxKeyEquivalent != null)
            {
                return child; // Don't overwrite
            }
            return child.keyEquivalent(key) 
        });   
        if (key != null)
        {
            cpy["gxKeyEquivalent"] = key
        } 
        return cpy;
    }

    /**
    * insets
    *
    * @param {number} l - left inset
    * @param {number} r - left inset
    * @param {number} t - top inset
    * @param {number} b - bottom inset
    */
    
	insets(l,t ,r,b) 
	{
		var cpy = this.copy();
		var insets = cpy["gxEdgeInsets"] || `0,0,0,0`; 
		insets = insets.split(",");
		insets = `${Number(insets[0]) + l},${Number(insets[1]) + t},${Number(insets[2]) + r},${Number(insets[3]) + b}`;
		cpy["gxEdgeInsets"] = insets;
		return cpy;
	}

	inset(inset)
	{
		return this.insets(inset,inset,inset,inset);
	}

	offset(x,y) 
	{
		var cpy = this.copy();
		var insets = cpy["gxEdgeInsets"] || "0,0,0,0"; 
		insets = insets.split(",");
		insets[0] -= x * -1;
		insets[1] -= y;
		insets[2] -= x;
		insets[3] -= y * -1;
		cpy["gxEdgeInsets"] = insets.join(",");
		return cpy;
	}

    loaded(loaded)
    {
        return loaded ? this : (new Base()).setValue(this.nsClass, "nsClass");
    }

    addStyle(styleName, newStyle)
    {
        var cpy = this.copy();         
        cpy[styleName] = newStyle.copy();  
        if (!cpy.addedStyles.includes(newStyle))
        {
            cpy.addedStyles.push(newStyle);
        }  
        return cpy;
    } 

    addStyles(styles, addition)
    {
        var cpy = this.copy()
        for (var s in styles)
        {
            cpy = cpy.addStyle(styles[s], addition.copy())
        }
        return cpy
    }

    styleByMutating(styleName, mutate)
	{ 
		var cpy = this.copy();
		cpy[styleName] = mutate(cpy);  
		return cpy;
	}

    // A better name than addStyle?
    state(stateName, newState)
    { 
        return this.addStyle(stateName, newState);
    }

}  

class BaseView extends Base 
{ 
    constructor()
    { 
        super();
        this.nsClass = "GXDrawView";  
        this.gxChildViews = [];
        this._gxChildren = [];
        this.gxLayout = [];   
        this.alignmentString = "";
        this.gxViewNameSeed = "";
        this.paddingValues = [];   
        this.gxIntrinsicHeight = -1;
        this.gxIntrinsicWidth = -1;
    }  
    
    lView() { this.getViewName(); return this.copy(); }
    rView() { this.getViewName(); return this.copy(); }
    tView() { this.getViewName(); return this.copy(); }
    bView() { this.getViewName(); return this.copy(); }

    wView() { this.getViewName(); return this.copy(); }
    hView() { this.getViewName(); return this.copy(); } 
    
    leftAnchor()    { return `[${this.lView().getViewName()}]`; };
    rightAnchor()   { return `[${this.rView().getViewName()}]`; };
    topAnchor()     { return `[${this.tView().getViewName()}]`; };
    bottomAnchor()  { return `[${this.bView().getViewName()}]`; };

    autoLoadConstraints() { return this.setValue(true , "gxAutoLoadConstraints") }

    viewsForSuperview()
    { 
        var views = [];
        if (this.lView().getViewName() != this.getViewName())
        {
            views.push(this.lView()); 
        } 
        if (this.rView().getViewName() != this.getViewName())
        {
            views.push(this.rView()); 
        }  
        if (this.tView().getViewName() != this.getViewName())
        {
            views.push(this.tView()); 
        } 
        if (this.bView().getViewName() != this.getViewName())
        {
            views.push(this.bView()); 
        } 
        views.push(this.copy());
        return views;
    }

    layoutForSuperview()
    {
        var layout = [];

        if (this.paddingValues[0] && this.paddingValues[0] != 0)
        { 
            layout.push(`H:${this.leftAnchor()}-${this.paddingValues[0]}-[${this.getViewName()}]`);   
        }
        if (this.paddingValues[1] && this.paddingValues[1] != 0)
        { 
            layout.push(`H:[${this.getViewName()}]-${this.paddingValues[1]}-${this.rightAnchor()}`);   
        }
        if (this.paddingValues[2] && this.paddingValues[2] != 0)
        { 
            layout.push(`V:${this.topAnchor()}-${this.paddingValues[2]}-[${this.getViewName()}]`);   
        }
        if (this.paddingValues[3] && this.paddingValues[3] != 0)
        { 
            layout.push(`V:[${this.getViewName()}]-${this.paddingValues[3]}-${this.bottomAnchor()}`);   
        } 
        return layout
    }
    
    viewName(newName)
    {
        return this.setValue(newName,"gxViewName");
    }
    
    width(w, priority = 1000) 
    {
        var cpy = this.copy();
        cpy.gxIntrinsicWidth = -1;
        if (cpy.lastWidth != null)
        {
            cpy.gxLayout = cpy.gxLayout.filter(e => e != cpy.lastWidth); 
        }
        cpy.lastWidth = `H:[self(${w}@${priority})]`
        cpy.gxLayout.push(cpy.lastWidth);
        return cpy; 
    }
    
    height(h, priority = 1000)
    {
        var cpy = this.copy();
        cpy.gxIntrinsicHeight = -1;
        if (cpy.lastHeight != null)
        {
            cpy.gxLayout = cpy.gxLayout.filter(e => e != cpy.lastHeight); 
        }
        cpy.lastHeight = `V:[self(${h}@${priority})]`
        cpy.gxLayout.push(cpy.lastHeight);
        return cpy;
    }
    
    
    size(w,h = w)
    { 
        return this.copy().width(w).height(h);
    } 
    
    fill(color)
    { 
        var cpy = this.copy();
        cpy.gxBackPainter = {
            nsClass: "GXRoundRectPainter",
            gxFillColor: color,
        }
        return cpy;
    }

    addToSuperview(sv)
    {     
        sv.addSubView(this);
    }

    willAddToSuperview(sv)
    { 
    }

    addSubView(view)
    { 
        view.willAddToSuperview(this);
        
        this.gxChildViews = this.gxChildViews.concat(view.viewsForSuperview())
        this.gxLayout = this.gxLayout.concat(view.layoutForSuperview());
        this._gxChildren = this.gxChildViews;
    }
    
    getViewName()
    { 
        if (this.gxViewName == null)
        {
            this.gxViewName = this.gxViewNameSeed + uuidv4();
        }
        return this.gxViewName
    }

    addLayout(layout)
    {
        var cpy = this.copy();
        cpy.gxLayout.push(layout);
        return cpy;
    }
    
    padding(l = 10, r = l,t = l ,b = l)
    {  
        var cpy = this.copy(); 
        cpy.paddingValues = [l,r,t,b];  
        cpy.getViewName(); // Cache view name
        if (l != 0)
        {
            cpy.lView = function() { return AnchorView().viewName("_left_" + this.getViewName()); };
        }
        if (r != 0)
        {
            cpy.rView = function() { return AnchorView().viewName("_right_" + this.getViewName()); };
        }
        if (t != 0)
        {
            cpy.tView = function() { return AnchorView().viewName("_top_" + this.getViewName()); };
        }
        if (b != 0)
        {
            cpy.bView = function() { return AnchorView().viewName("_bottom_" + this.getViewName()); };
        }
        return cpy;
    } 
    
    forePainter(painter)
    { 
        return this.setValue(painter, "gxForePainter");
    }
    iconPainter(painter)
    { 
        return this.setValue(painter, "gxIconPainter");
    }
    backPainter(painter)
    {
        return this.setValue(painter,"gxBackPainter");
    }

    caarPainter(painter) 
    {
        return this.setValue(painter,"gxCaarPainter");
    } 

    textPainter(painter)
    {
        return this.setValue(painter,"gxTextPainter");
    }
    extraPainter(painter)
    {
        return this.setValue(painter,"gxExtraPainter");
    }
    imagePainter(painter)
    {
        return this.setValue(painter,"gxImagePainter");
    }

    knobPainter(painter)
	{ 
		return this.setValue(painter,"gxKnobPainter");
    } 

	knobPainterExtra(painter)
	{ 
		return this.setValue(painter,"gxKnobPainterExtra");
	}

	detentPainter(painter)
	{
		return this.setValue(painter, "gxDetentPainter")
	}

    toolTipKeyPath(keyPath)
    {
        return this.setValue(keyPath, "gxToolTipKeyPath");
    }
    bordered(color = Colors.warning)
    {
        return this.backPainter(Painter().stroke(2, color));
    }
    
    frame(frameRect)
    {
        return this.setValue(frameRect,"gxFrameRect");
    }
    
    intrinsicSize(w,h = w)
    {
        return this.copy().setValues({
            gxIntrinsicWidth: w,
            gxIntrinsicHeight: h, 
        });
    }
    
    fillHeight(priority = 450)
    {  
        return this.addLayout(`V:|-0@${priority}-[self]-0@${priority}-|`);
    }
    
    fillWidth(priority = 450)
    {
        return this.addLayout(`H:|-0@${priority}-[self]-0@${priority}-|`);
    }
    
    fillFrame(priority = 450)
    {
        return this.fillWidth(priority).fillHeight(priority)
    }
    
    pinLeft()
    {
        return this.leadingPad("H",0);
    }
    
    pinRight()
    {
        return this.trailingPad("H",0);
    }
    
    pinTop()
    {
        return this.leadingPad("V",0);
    }
    
    pinBottom()
    {
        return this.trailingPad("V",0);
    }
    
    fillParent(priority = 450)
    {
        return this.fillWidth(priority).fillHeight(priority);
    }
    
    centerV()
    {
        return this.addLayout("H:[superview]-(<=0)-[self]&alignCenterY")
    }
    
    centerH()
    {
        return this.addLayout("V:[superview]-(<=0)-[self]&alignCenterX")
    }

    center()
    {
        return this.centerH().centerV();
    }
    
    leadingPad(orientaion, amt)
    {
        return this.addLayout(`${orientaion}:|-(${amt})-[self]`)
    }
    
    trailingPad(orientaion, amt)
    {
        return this.addLayout(`${orientaion}:[self]-(${amt})-|`)
    }
    
    leftPad(amt)
    {
        return this.addLayout(`H:|-(${amt})-[self]`)
    }
    
    rightPad(amt)
    {
        return this.addLayout(`H:[self]-(${amt})-|`)
    }
    
    topPad(amt)
    {
        var cpy = this.copy(); 
        var lastLayout = cpy.topPadLayout ? cpy.topPadLayout : "V:|-(0)-[self]";
        var newLayout = `V:|-(${amt})-[self]`;
        cpy.topPadLayout = newLayout;
        cpy["gxLayout"] = cpy["gxLayout"].filter(e => e != lastLayout); 
        return cpy.addLayout(newLayout)
    }
    
    bottomPad(amt)
    {
        return this.addLayout(`V:[self]-(${amt})-|`)
    }
    
    hPad(pad) 
    {
        var cpy = this.copy();
        // remove layouts that will conflict
        cpy["gxLayout"] = cpy["gxLayout"].filter(e => e != "H:|-0-[self]-0-|"); 
        return cpy.addLayout(`H:|-(${pad})-[self]-(${pad})-|`);
    }
    
    vPad(pad) 
    { 
        return this.topPad(pad).bottomPad(pad);
    }

    shrinky()
    {
        return this
            .addLayout("V:[self(<=1@100)]")
            .addLayout("H:[self(<=1@100)]");
    }
    
    label(label, leading = true, spacing = 5, align="alignCenterY")
    {
        var cpy = this.copy();
        cpy.getViewName();
        cpy["gxLabelTemplate"] =  ReadoutLabel().height(24).setValues({
            "gxDisableOnKey" : cpy.gxDisableOnKey,
            "gxEnableOnKey" : cpy.gxEnableOnKey,
        }) 
        cpy["gxLabelText"] = label; 
        if (leading)
        {
            cpy.leftAnchor =  function() { return `[${cpy.getViewName() + "Label"}]`; };
            cpy.rightAnchor =  function() { return `[${cpy.getViewName()}]`; }; 
            cpy = cpy.addLayout(`H:[selfLabel]-${spacing}-[self]&${align}`);
        } 
        else 
        {
            cpy.rightAnchor =  function() { return `[${cpy.getViewName() + "Label"}]`; };
            cpy.leftAnchor =  function() { return `[${cpy.getViewName()}]`; };
            cpy = cpy.addLayout(`H:[self]-${spacing}-[selfLabel]&${align}`);
        } 
        return cpy
    }
    
    readout(path)
    {
        var cpy = this.copy();
        cpy.getViewName(); 
        cpy["gxReadoutTemplate"] =  Label().textAlign(1,0.5).fontColor(Colors.fontColor).fontSize(13); 
        cpy["gxReadoutKeyPath"] = path ? path : cpy.gxValueKeyPath; 
        cpy.rightAnchor = function() { return `[${cpy.getViewName() + "Readout"}]`; }; 
        return cpy.addLayout("H:[self]-(5.0)-[selfReadout]&alignCenterY");
    }
    
    readoutFormatter(formatter)
    {
        var cpy = this.copy().readout(this["gxReadoutKeyPath"] ? this["gxReadoutKeyPath"] : this.gxValueKeyPath);
        cpy["gxReadoutTemplate"] = cpy["gxReadoutTemplate"].formatter(formatter).setValues({
            "gxDisableOnKey" : cpy.gxDisableOnKey,
            "gxEnableOnKey" : cpy.gxEnableOnKey,
        }) 
        return cpy;
    }
    
    baseline(offset)
    {
        return this.setValue(offset, "gxBaselineOffset");
    }
	
    layerZ(layer)
    {
        var cpy = this.copy();
        cpy["gxLayerZPosition"] = layer;
        return cpy;
    } 
    
    backgroundFill(color)
    {
        return this.backPainter(Painter().fill(color));
    } 
    
    toolTip(tooltip, pressedTooltip = tooltip)
    {
        var cpy = this.copy();
        cpy["gxToolTip"] = tooltip;
        cpy["gxPressedToolTip"] = pressedTooltip;
        return cpy;
    }
    
    showKey(key)
    {
        var cpy = this.copy();
        cpy.gxValueKeyPath = cpy.gxValueKeyPath || ""; // Makes sure this view always gets updates
        cpy["gxShowOnKey"] = key;
        return cpy;
    }
    
    hideKey(key)
    {
        var cpy = this.copy();
        cpy.gxValueKeyPath = cpy.gxValueKeyPath || ""; // Makes sure this view always gets updates
        if (key == null)
        {
            cpy.gxHideOnNilValue = true
        }  
        else 
        {
            cpy["gxHideOnKey"] = key;
        }
        return cpy;
    }
    
    collapseKey(key, horizontal = false)
    {
        return VList([
            this.copy().hideKey(key).fillWidth().fillHeight()
        ]).ignoreAX().setValues({ 
            gxLayoutHorizontally : horizontal
        });
    }

    expandKey(key, horizontal = false)
    {
        return VList([
            this.copy().showKey(key).fillWidth().fillHeight()
        ]).ignoreAX().setValue(horizontal, "gxLayoutHorizontally")
    }
    
    enableKey(key)
    {
        var cpy = this.forAllChildren((child) => { 
            if (child.gxEnableOnKey != null)
            {
                return child; // Don't overwrite
            }
            return child.enableKey(key) 
        });    
        cpy["gxValueKeyPath"] = cpy["gxValueKeyPath"] || ""; // Makes sure this view always gets updates
        cpy["gxEnableOnKey"] = key;
        return cpy;
    }
    
    disableKey(key)
    { 
        var cpy = this.forAllChildren((child) => { 
            if (child.gxDisableOnKey != null)
            {
                return child; // Don't overwrite
            }
            return child.disableKey(key) 
        });    
        cpy["gxValueKeyPath"] = cpy["gxValueKeyPath"] || ""; // Makes sure this view always gets updates
        cpy["gxDisableOnKey"] = key;
        return cpy; 
    }
    
    animateKey(key)
    {
        var cpy = this.copy();
        cpy["gxValueKeyPath"] = cpy["gxValueKeyPath"] || ""; // Makes sure this view always gets updates
        cpy["gxAnimateOnKey"] = key;
        return cpy;
    }
    
    animateVisibility(speedIn, speedOut = speedIn)
    {
        var cpy = this.copy();
        cpy["gxShowSpeedSeconds"] = speedIn;
        cpy["gxHideSpeedSeconds"] = speedOut;
        return cpy;
    }
    
    stylesKeyPath(path)
    {
        return this.setValue(path, "gxStylesKeyPath")
    }

    optionalRecursiveForAllChildren(recurse, action)
    {
        if (!recurse)
        {
            return this;
        }
        return this.forAllChildren((child) => { return child.forAllChildren(action) });
    }
    
    ignoreAX(recurse)
    {
        var cpy = this.optionalRecursiveForAllChildren(recurse, (child) => { return child.ignoreAX(recurse) });  
        
        cpy = cpy.setValue(true, "axIsIgnored");
        cpy.setAXTitle      = function(v) { return this.copy(); };
        cpy.setAXFormatter  = function(v) { return this.copy(); };
        cpy.setAXHelp       = function(v) { return this.copy(); };
        cpy.setAXHelpPath   = function(v) { return this.copy(); };
        cpy.setAXRole       = function(v) { return this.copy(); };
        return cpy;
    }
    
    setAXTitle(title, recurse = false)
    { 
        if (title == null)
            return this;
        var cpy = this.optionalRecursiveForAllChildren(recurse, (child) => { return child.setAXTitle(title) });  
        if (cpy.axTitle != null)
        {
            return cpy;
        }
        return cpy.setValues({
            "axIsIgnored" : false,
            "axTitle" : title,
        });
    }

    setAXDescription(desc, recurse = false)
    { 
        if (desc == null)
            return this;
        var cpy = this.optionalRecursiveForAllChildren(recurse, (child) => { return child.setAXDescription(title) });  
        if (cpy.axDescription != null)
        {
            return cpy;
        }
        return cpy.setValues({
            "axIsIgnored" : false,
            "axDescription" : title,
        });
    }

    setAXFormatter(formatter, recurse = false)
    { 
        if (formatter == null)
            return this;
        var cpy = this.optionalRecursiveForAllChildren(recurse, (child) => { return child.setAXFormatter(formatter) });  
        if (cpy.axFormatter != null)
        {
            return cpy;
        }
        return cpy.setValues({
            "axIsIgnored" : false,
            "axFormatter" : formatter,
        });
    }
    
    setAXHelp(help, recurse = false)
    {
        if (help == null)
            return this;
        var cpy = this.optionalRecursiveForAllChildren(recurse, (child) => { return child.setAXHelp(help) });  
        if (cpy.axHelp != null)
        {
            return cpy;
        }
        return cpy.setValues({
            "axIsIgnored" : false,
            "axHelp" : help,
        });
    }
    
    setAXHelpPath(keyPath, recurse = false)
    {
        if (keyPath == null)
            return this;
        var cpy = this.optionalRecursiveForAllChildren(recurse, (child) => { return child.setAXHelpPath(keyPath) }); 
        if (cpy.axHelpKeyPath != null)
        {
            return cpy;
        } 
        return cpy.setValues({
            "axIsIgnored" : false,
            "axHelpKeyPath" : keyPath,
        });
    }

    setAXRole(role, recurse = false)
    { 
        if (role == null)
            return this;
        var cpy = this.optionalRecursiveForAllChildren(recurse, (child) => { return child.setAXRole(role) });  
        if (cpy.axRoleDescription != null)
        {
            return cpy;
        } 
        return cpy.setValues({
            "axIsIgnored" : false,
            "axRoleDescription" : role,
        });
    } 

    checkBoxRole(recurse = false)
	{
        var cpy = this.optionalRecursiveForAllChildren(recurse, (child) => { return child.checkBoxRole() });  
        if (cpy.axCheckboxRole != null)
        {
            return cpy;
        } 
        return cpy.setValues({ 
            axCheckboxRole: true
        });
	} 
    
    noKey()
    {
        return this.setValue(false, "gxCanBecomeKeyView");
    }

    flipped(isFlipped = true)
    {
        return this.setValue(!isFlipped, "gxViewFlipped")
    }
    
    nextKey(view)
    {
        return this.setValue(view, "gxNextKeyViewName")
    }
    
    wantsLayer(layer = true)
    {
        return this.setValue(layer, "gxWantsLayer")
    }
    
    visible(isVisible)
    {
        if (isVisible)
        {
            return this.copy();
        }
        else
        {
            return View().className(this.nsClass);
        }
    }
    
    background(view)
    {
        var bgView = view.copy();
        var thisView = this.copy(); 
        
        var stack = ZStack([ 
            bgView,
            thisView,
        ]);

        stack.stackLayout.push(`V:[${bgView.getViewName()}]-0-${thisView.leftAnchor()}&alignLeading`);
        stack.stackLayout.push(`V:[${bgView.getViewName()}]-0-${thisView.rightAnchor()}&alignTrailing`);
        stack.stackLayout.push(`H:[${bgView.getViewName()}]-0-${thisView.topAnchor()}&alignTop`);
        stack.stackLayout.push(`H:[${bgView.getViewName()}]-0-${thisView.bottomAnchor()}&alignBottom`);

        return stack.makeView();  
    }
    
    painterByMutating(painter, mutate)
    { 
        var cpy = this.copy();
        cpy[painter] = mutate(cpy[painter]);  
        return cpy;
    }

    childViews(views)
    {
        if (this.isStackView)
        {
            return this.copy();
        }  
        var stack = ZStack(views).makeView();
        var other = this.copy();
        other.gxChildViews = other.gxChildViews.concat(stack.gxChildViews);
        other._gxChildren = other.gxChildViews;
        other.gxLayout = other.gxLayout.concat(stack.gxLayout);
        return Object.assign(stack, other);
    } 

    subLayers(layers)
    { 
        return this.setValue(layers,"gxSublayers")
    }

    menuItems(items)
    {
        return this.setValue(items, "gxMenuItems")
    }

    action(action)
    {
        return this.setValue(action,"gxAction", true)
    }

    onView(view)
    {
        return ZStack([
            view.copy().fillFrame().showKey(this.gxValueKeyPath),
            this.copy()
        ]).makeView().keyPath(this.gxValueKeyPath).wantsLayer()
    }

    offView(view)
    {
        return ZStack([
            view.copy().fillFrame().hideKey(this.gxValueKeyPath),
            this.copy()
        ]).makeView().keyPath(this.gxValueKeyPath).wantsLayer()
    }
    
    pressView(view)
    {
        return ZStack([ 
            this.copy(),
            view.copy().className("GXButtonView").fillFrame().ignoreAX(),
        ]).makeView().keyPath(this.gxValueKeyPath).wantsLayer()
    } 

    wrapInFlexibleContainer()
    {
        return ZStack([
            this.copy()
        ]).align("center");
    }

    makeView(name = null)
    { 
        return this.copy()
    }

    formatter(fmt)
    {
        var cpy = this.copy()  
        return cpy.setValue(fmt, "gxFormatter")
    }

    padding2(l=10,r=10,t=10,b=10)
    { 
        return HStack([
            Padding(l),
            VStack([
                Padding(t),
                this.copy(),
                Padding(b)
            ]),
            Padding(r)
        ]);
    }
    
}

class BaseLayer extends BaseView 
{
    constructor()
    { 
        super();
        this.nsClass = "GXDrawLayer";  
        this.gxSublayers = [];   
        this.gxContentsGravity = "resize"; 
        this.gxRendersTransform = true;
    } 
    
    mask(layer)
    {
        var l = (layer || Layer().backPainter(Painter().fill(Colors.black)));
        return this.setValue(l, "gxMaskLayer");
    }

    fillColor(c)
	{ 
		return this.setValue(c, "gxFillColor");
	}

    opacity(v)
	{ 
		return this.setValue(v, "gxOpacity");
	} 

    makeView()
    {
        return LayerView([
            this.copy().fillFrame(),
        ]);
    } 
} 

class BaseStackView extends BaseView
{
    constructor()
    {
        super(); 
        this.isStackView = true; 
        this.stackViews = [];  
        this.stackLayout = [];
        this.alignmentDirection = "V";
        this.gxViewNameSeed = "_STACK_"
        this.stackLayout.push(`H:${this.leftAnchor()}-0-[${this.wView().getViewName()}]-0-${this.rightAnchor()}`);
        this.stackLayout.push(`V:${this.topAnchor()}-0-[${this.hView().getViewName()}]-0-${this.bottomAnchor()}`); 
        this.gxIntrinsicHeight = -1;
        this.gxIntrinsicWidth = -1;
        this.allViews =     []; 
    } 
    
    lView() { return AnchorView().viewName("_left_" + this.getViewName()) }
    rView() { return AnchorView().viewName("_right_" + this.getViewName()) }
    tView() { return AnchorView().viewName("_top_" + this.getViewName()) }
    bView() { return AnchorView().viewName("_bottom_" + this.getViewName()) }   
    wView() { return AnchorView().viewName("_width_" + this.getViewName()) }
    hView() { return AnchorView().viewName("_height_" + this.getViewName()) }

    viewsForSuperview() 
    { 
        return this.allViews.concat([this.lView(), this.rView(), this.tView(), this.bView(), this.wView(), this.hView()]); 
    }

    layoutForSuperview()
    {
 
        var layout = []; 
    
        layout = layout.concat(this.stackLayout);

        if (this.alignmentString != "")
        {
            layout.push(this.alignmentString)
        }  

        return layout;
    }
     
    align(alignment = "center")
    { 
        var cpy = this.copy();
        cpy.alignmentString = "";
       
        if (alignment)
        {  
            if (alignment == "center")
            {
                if (cpy.alignmentDirection == "H")
                {
                    alignment = "centerY";
                } 
                else 
                {
                    alignment = "centerX";
                }
            }  
            
            alignment = alignment.charAt(0).toUpperCase() + alignment.slice(1);
            
            cpy.alignmentString = "";

            var hasAlignment = false 
           
            for (var i in cpy.stackViews)
            {
                let view = cpy.stackViews[i]; 
                  
                hasAlignment = true; 

                if (alignment == "CenterX")
                {
                    if (i == 0)
                    {
                        cpy.alignmentString = `[${cpy.wView().getViewName()}]-(>=0)-`
                    }
                    cpy.alignmentString += `[${view.wView().getViewName()}]-(>=0)-`;
                }
                else if (alignment == "CenterY")
                {
                    if (i == 0)
                    {
                        cpy.alignmentString = `[${cpy.hView().getViewName()}]-(>=0)-`
                    }
                    cpy.alignmentString += `[${view.hView().getViewName()}]-(>=0)-`;
                }
                else if (alignment == "Top")
                {
                    if (i == 0)
                    { 
                        cpy.alignmentString = `${cpy.topAnchor()}-(>=0)-`
                    }
                    cpy.alignmentString += `${view.topAnchor()}-(>=0)-`;
                }
                else if (alignment == "Bottom")
                {
                    if (i == 0)
                    {
                        cpy.alignmentString = `${cpy.bottomAnchor()}-(>=0)-`
                    }
                    cpy.alignmentString += `${view.bottomAnchor()}-(>=0)-`;
                }
                else if (alignment == "Left" || alignment == "Leading")
                {
                    if (i == 0)
                    {
                        cpy.alignmentString = `${cpy.leftAnchor()}-(>=0)-`
                    }
                    cpy.alignmentString += `${view.leftAnchor()}-(>=0)-`; 
                }   
                else if (alignment == "Right" || alignment == "Trailing")
                {
                    if (i == 0)
                    {
                        cpy.alignmentString = `${cpy.rightAnchor()}-(>=0)-`
                    }
                    cpy.alignmentString += `${view.rightAnchor()}-(>=0)-`;
                } 
                else if (alignment == "Baseline")
                {
                    if (i == 0)
                    {
                        cpy.alignmentString = `${cpy.leftAnchor()}-(>=0)-`
                    }
                    cpy.alignmentString += `[${view.wView().getViewName()}]-(>=0)-`;
                }
            }
            if (hasAlignment)
            { 
                cpy.alignmentString = cpy.alignmentString.slice(0, -7); 
                cpy.alignmentString = `${cpy.alignmentDirection}:` + cpy.alignmentString + `&align${alignment}`;   
            } 
        }    
        return cpy;
    }

    addSubView(view)
    { 
        view.willAddToSuperview(this);

        this.stackLayout = this.stackLayout.concat(view.layoutForSuperview());
        this.allViews    = this.allViews.concat(view.viewsForSuperview());  
       
        this.stackViews.push(view.copy()); 
    }
     
    padding(pad = 10)
    {  
        var cpy = this.copy();

        cpy.lView = function() { return AnchorView().viewName("_left_"   + this.getViewName()).width(pad) };
        cpy.rView = function() { return AnchorView().viewName("_right_"  + this.getViewName()).width(pad); };
        cpy.tView = function() { return AnchorView().viewName("_top_"    + this.getViewName()).height(pad); };
        cpy.bView = function() { return AnchorView().viewName("_bottom_" + this.getViewName()).height(pad); };  
        
        return cpy;
    } 
    
    spaceViews(views = [], priority = 1000)
    {
        var cpy = this.copy();   

        if (views.length == 0)
        {
            for (var v in cpy.stackViews)
            {
                views.push(cpy.stackViews[v].getViewName());
            }
        }

        if (views.length == 0)
        {
            return cpy;
        }

        var spacerViews = [];

        for (var v in views)
        {   
            let viewName = views[v];
            for (var s in cpy.stackViews)
            {
                let stackView = cpy.stackViews[s];
                if (stackView.getViewName() == viewName)
                {
                    spacerViews.push(stackView);
                }
            }
        } 

        var firstView = spacerViews[0].wView().getViewName();

        for (var v in spacerViews)
        { 
            if (v == 0) { continue; }
            cpy.stackLayout.push(`${this.alignmentDirection}:[${spacerViews[v].wView().getViewName()}(==${firstView}@${priority})]`);
        }
        return cpy;
    }

    makeView(name = null)
    { 
        var newView = View();
        var cpy = this.copy();   

        newView.gxChildViews    = cpy.viewsForSuperview(); 
        newView.gxLayout        = cpy.layoutForSuperview();  
        newView.gxValueKeyPath  = cpy.gxValueKeyPath;
        newView.gxHideOnKey     = cpy.gxHideOnKey;
        newView.gxShowOnKey     = cpy.gxShowOnKey;

        if (cpy.hView().getViewName() != cpy.getViewName())
        {
            newView.gxLayout.push(`V:|[${cpy.hView().getViewName()}]|`);
            newView.gxLayout.push(`H:|[${cpy.wView().getViewName()}]|`);
        }    
        return newView.className(name).intrinsicSize(-1, -1);
    }     
}


class BaseZStackView extends BaseStackView
{ 
    constructor()
    {
        super(); 
    }

    align(alignment = "center")
    {
        // Handle cases other than "center" via the superclass
        if (alignment != "center")
        {
            if (alignment == "top" || alignment == "bottom" )
            {
                this.alignmentDirection  = "H";
            }
            else 
            {
                this.alignmentDirection = "V";
            } 
            return super.align(alignment);
        }

        // Center all views both vertically and horizonatally
        var cpy = this.copy();
             
        var layoutStringH = `V:[${cpy.wView().getViewName()}]`;
        var layoutStringV = `H:[${cpy.hView().getViewName()}]`;

        for (var i in cpy.stackViews)
        {
            var view = cpy.stackViews[i];
            if (view.isStackView)
            {
                layoutStringH = layoutStringH + `-(<=0)-[${view.wView().getViewName()}]`; 
                layoutStringV = layoutStringV + `-(<=0)-[${view.hView().getViewName()}]`; 
            }
            else 
            {
                layoutStringH = layoutStringH + `-(<=0)-[${view.getViewName()}]`; 
                layoutStringV = layoutStringV + `-(<=0)-[${view.getViewName()}]`; 
            } 
        }

        if (cpy.stackViews.length > 0)
        {
            var layoutStringH = layoutStringH + "&alignCenterX";
            var layoutStringV = layoutStringV + "&alignCenterY";
            cpy.stackLayout.push(layoutStringV);
            cpy.stackLayout.push(layoutStringH);
        }
    
        return cpy;
    }   
    
}


class BaseLabelView extends BaseView
{
	constructor(text = "")
	{
		super();
        this.nsClass = "FGLabelView";  
		this.gxForePainter = LabelPainter().color(Colors.primaryContent).align(0,0.5);
		this.gxDefaultValue = text;
        this.gxViewNameSeed = "_LABEL_";
		this.axIsIgnored = true;
		this.gxCanBecomeKeyView = false;
        this.gxUseInlineMarkup = false;
        delete this.gxIntrinsicHeight;
        delete this.gxIntrinsicWidth; 
	} 
    
	fontSize(size)
	{
		var cpy = this.copy() 
		cpy.gxForePainter = cpy.gxForePainter.fontSize(size);
		return cpy;
	}

	fontColor(color)
	{
		var cpy = this.copy()  
		cpy.gxForePainter = cpy.gxForePainter.color(color);
		return cpy;
	}

	fontWeight(w)
	{
		var cpy = this.copy()
		cpy.gxForePainter = cpy.gxForePainter.weight(w);
		return cpy;
    }
    
    clickValue(val)
    {
        var cpy = this.copy()
		cpy.gxClickValue = val
		return cpy;
    }

	// weight (font weights match apple's naming)
	light() { return this.fontWeight(-0.4) }
	regular() { return this.fontWeight(0) }
	medium() { return this.fontWeight(0.23) }
	bold() { return this.fontWeight(0.4) }
	left() { return this.textAlign(0,this["gxVAlign"]) }
	right() { return this.textAlign(1,this["gxVAlign"]) }
	center() { return this.textAlign(0.5,this["gxVAlign"]) }
	top() { return this.textAlign(this["gxHAlign"],1) }
	bottom() { return this.textAlign(this["gxHAlign"],0) }
    centerY() { return this.textAlign(this["gxHAlign"],0.5) }
    centerX() { return this.textAlign(0.5,this["gxVAlign"]) }
	
	textAlign(x,y)
	{
		var cpy = this.copy()
		cpy.gxForePainter = cpy.gxForePainter.align(x,y);
		return cpy;
	} 
    
    lineBreakMode(val)
    {
        var cpy = this.copy()
        cpy.gxForePainter = cpy.gxForePainter.lineBreakMode(val) 
        return cpy
    }
    
    useInlineMarkup(val = true)
    {
        var cpy = this.copy()
        cpy.gxForePainter["gxUseInlineMarkup"] = val
        return cpy
    } 
}

class BaseListView extends BaseView
{
    constructor()
    {
        super();
        this.nsClass = "Farrago.FGListView";
        this.gxValueKeyPath = ""; // Always update 
		this.gxLayoutSubviews = true;
		this.axIsIgnored = true;
        this.gxViewNameSeed = "_LIST_";
		this.gxCollapsible = true;
		this.gxLayoutHorizontally = false;  
    }    
 
	topPad(p)
	{
		let cpy = this.copy()
		cpy["listTopHeight"] = p;
		return cpy;
    }
    
	bottomPad(p)
	{
		let cpy = this.copy()
		cpy["listBottomHeight"] = p;
		return cpy;
    }
    
	minimumHeight(p)
	{
		let cpy = this.copy()
		cpy["listMinimumHeight"] = p;
		return cpy;
    }
    
	rowSpacing(spacing)
	{
		let cpy = this.copy()
		cpy["listItemSpacing"] = spacing;
		return cpy;
    } 

    addSubView(view)
    {
        view.willAddToSuperview(this);
        this.gxChildViews = this.gxChildViews.concat(view.viewsForSuperview());
    }
}
    
class BasePainter extends Base
{  

	constructor ()
	{
		super();
		this.gxHAlign = 0.5;
		this.gxVAlign = 0.5; 
	}
	
	fill(fillColor, fillEnd = fillColor) 
	{ 
		return this.setValues({
            gxFillColor: fillColor,
            gxFillEndColor: fillColor !== fillEnd ? fillEnd : null
        });
	}

	radius(radius) 
	{
		return this.setValue(radius, "gxCornerRadius");
	}

	radii(tl,tr,bl,br) 
	{
		return this.setValue(`${tl},${tr},${br},${bl}`, "gxCornerRadii");  
	}

	align(x,y)
	{
		var cpy = this.copy();
        cpy["gxAlignment"] = `{${x},${y}}`
		cpy["gxHAlign"] = x;
		cpy["gxVAlign"] = y;
		return cpy;
    } 

	stroke(width = 0, color)
	{
		var cpy = this.copy();
		cpy["gxStrokeWidth"] = width;
		cpy["gxFrameColor"] = color;
		return cpy;
	}  
 
    innerShadow()
    {
        return this.setValue(true, "gxInnerShadow")
    }

	frame(x,y,w,h) 
	{ 
		return this.setValue(`${x},${y},${w},${h}`, "gxFrameRect");
	}

	size(w,h = w) 
	{ 
		return this.setValue(`{${w},${h}}`, "gxDefaultSize");
	} 

	tintColor(tint) 
	{
		var cpy = this.copy()
		cpy["gxTintColor"] = tint;
		return cpy;
	}
  
	color(c) 
	{
		var cpy = this.copy();
		cpy.gxFontColor = c;
		return cpy;
	} 

	weight(w) 
	{
		var cpy = this.copy();
		cpy.gxFontWeight = w;
		return cpy;
	}

	fontSize(s)
	{
		return this.setValue(s, "gxFontSize");
	}

    fontFamily(s)
	{
		return this.setValue(s, "gxFontFamily");
	}

    size(w,h=w)
    {
        return this.setValue(`${w},${h}`, "gxDefaultSize");
    }

	// weight (font weights match apple's naming)
	light() { return this.weight(-0.4) }
	regular() { return this.weight(0) }
	medium() { return this.weight(0.23) }
	bold() { return this.weight(0.4) }

	monospacedDigits(mono)
	{
		return this.setValue(mono, "gxMonoSpacedDigits");
	}
	
    lineBreakMode(val)
    {
        var cpy = this.copy() 
        return cpy.setValue(val, "gxLineBreakMode")
    } 

    addStyleWithPainter(styleName, cb)
    { 
        return this.addStyle(styleName, cb(this.copy()));   
    }

    addContrast()
    {
        return this.addStyleWithPainter("increasedContrast", (painter) => {
            return painter.stroke(1, Colors.contrastBorder)
        })
    }
}

class BaseButtonView extends BaseView 
{
	constructor(title)
	{
		super();
		this.nsClass = "FGButtonView";
		this.gxTitle = title; 
		this.gxLabelText = "";
		this.gxPullsDown = false; 
	}  
	
	setTitle(title)
	{
		var cpy = this.copy();
		cpy["gxTitle"] = title;
		return cpy;
    }
    
    titleKeyPath(path)
    {
        var cpy = this.copy();
		cpy["gxTitleKeyPath"] = path;
		return cpy;
    }

    iconKeyPath(path)
    {
        var cpy = this.copy();
		cpy["gxIconKeyPath"] = path;
		return cpy;
    }
	
	pullsDown()
	{
		var cpy = this.copy();
		cpy["gxPullsDown"] = true;
        return cpy;
        
	}
    
	menuItemsPath(keyPath)
	{
		var cpy = this.copy();
		cpy["gxMenuItemsKeyPath"] = keyPath;
		return cpy;
	}
    
	drawArrows(imageName = "popup.arrows", color = Colors.primaryContent)
	{
		var cpy = this.copy();
		cpy = cpy.extraPainter(ImagePainter(imageName).align(1,0.5).offset(-10,0).tintColor(color));
		cpy["gxForePainter"] = cpy["gxForePainter"].insets(0,0,20,0);
		return cpy; // Inset right edge of title to account for arrows
	} 
 
}


class BaseColor extends Base 
{
    constructor(h = 100, s = 100, b = 100, a = 100)
    {
        super();
        this.nsClass = "NSColor";
        this.gxHue = h;
        this.gxSaturation = s;
        this.gxBrightness = b;
        this.addedStyles = [];
        this.gxAlpha = a;
    }

    withAlpha(val) 
    {
        var c = this.copy()
        c.gxAlpha *= val;
        return c;
    }  

    brightness(val)
    {
        var c = this.copy() 
        c.gxBrightness *= val;
        return c;
    }

    saturation(val)
    {
        var c = this.copy() 
        c.gxSaturation *= val;
        return c;
    } 
  
    showPressed()
    {
        return this.addStyle("pressed", this.withAlpha(0.7))
    }
    
    showDisabled(color)
    {
        if (color == null)
        {
            return this.addStyle("disabled", this.brightness(1.5))
        }
        return this.addStyle("disabled", color)
    }
    
    mutateAllStyles(mutate)
    {
        var cpy = this.copy();
        for (var i in cpy.addedStyles)
        {
            var style = cpy.addedStyles[i];
            cpy[style] = mutate(cpy[style]);
        }
        return cpy;
    }
}

class BaseSystemColor extends BaseColor
{
    constructor(systemColor, name)
    {
        super(100,100,100,100);
        this.nsClass = "GXAccentColor";
        this.gxColorModifierKey = name;
        this.gxColorName = systemColor; 
    }   
}

class BaseThemedColor extends BaseColor
{
    constructor(theme1Name, theme1Color, theme2Name = theme1Name, theme2Color = theme1Color)
    {
        super(0,0,0,100)    

        this.theme1Name = theme1Name;
        this.theme2Name = theme2Name; 

        this[theme1Name] = theme1Color;
        this[theme2Name] = theme2Color; 
    }

    inverted() 
    {
        return new BaseThemedColor(this.theme1Name, this[this.theme2Name], this.theme2Name, this[this.theme1Name]);
    }
    
    mutateAllStyles(mutate)
    {
        var cpy = this.copy();
        if (cpy[cpy.theme1Name]) 
        {
            cpy[cpy.theme1Name] = mutate(cpy[cpy.theme1Name]);
        }
        if (cpy[cpy.theme2Name])   
        {
            cpy[cpy.theme2Name] = mutate(cpy[cpy.theme2Name]);
        }
        for (var i in cpy.addedStyles)
        {
            var style = cpy.addedStyles[i];
            if (cpy[cpy.theme1Name][style])   
            {
                cpy[cpy.theme1Name][style] = mutate(cpy[cpy.theme1Name][style]);
            }
            if (cpy[cpy.theme2Name][style])   
            {
                cpy[cpy.theme2Name][style] = mutate(cpy[cpy.theme2Name][style]);
            } 
        }
        return cpy;
    }

	withAlpha(val) 
	{      
        return this.mutateAllStyles((style) => { return style.withAlpha(val); });
    }
    
	brightness(val) 
	{
        return this.mutateAllStyles((style) => { return style.brightness(val); });
    }
    
	saturation(val) 
	{
        return this.mutateAllStyles((style) => { return style.saturation(val); });
    }  
    
	showPressed() 
	{  
        return this.mutateAllStyles((style) => { return style.showPressed(); });
    }
     
	disabledColor(style1, style2) 
	{
		return this.addStyle("disabled", style1, style2 | style1);	
    }
     
	addStyle(newStyle, style1, style2 = style1)
	{    
        var cpy = this.copy();   
       
        if (!cpy.addedStyles.includes(newStyle))
        {
            cpy.addedStyles.push(newStyle);
        } 
        
        cpy[cpy.theme1Name] = cpy[cpy.theme1Name].setValues({  
            [newStyle] : style1[cpy.theme1Name] ? style1[cpy.theme1Name].copy() : style1.copy(),
        });
        
        cpy[cpy.theme2Name] = cpy[cpy.theme2Name].setValues({ 
            [newStyle] : style2[cpy.theme2Name] ? style2[cpy.theme2Name].copy() : style2.copy(),
        });  
        
        return cpy;  
    }   
}
