markup.push
({ 

	"EmojiItem" : View("FGEmojiItemView")  
		.keyPath("")
		.forePainter(LabelPainter().color(Colors.fontColor).fontSize(14).align(0.5,0.5))
		.childViews([
			Button()
				.keyPath("")
				.viewName("deleteButton")
				.size(14)
				.backPainter(Painter().radius(10).fill(Colors.tertiaryContent.withAlpha(0.6).showPressed()))
				.forePainter(ImagePainter("delete").tintColor(Colors.primaryContent.showPressed()))
				.centerH().centerV()
		])

});

function TileInspectorPanel()
{
	return VStack([ 
		// Fixed content
		MainContentView().fillWidth(),  
		VList([
			PlaybackPane().viewName("playbackPane").hideKey("tile.notesShowing"),
			VStack([
				NotesPane("Tile").hPad(15),
				Padding(), 
			])
			.makeView()
			.viewName("notesPane")
			.hideKey("tile.playbackShowing"),
		])
		.addLayout("V:[notesPane(>=playbackPane)]")
		.fillWidth()
		.ignoreAX(),
		AutomationSection("tileAutomationsList")
			.setAXTitle(`Tile Automations`)
			.setAXRole("Automation List")
			.fillWidth(), 
		Spacer(),
		// Replace file section 
		HStack([
			Padding(15),
			Label("File")
				.fontSize(13),
			Padding(15), 
			Label() 
				.lineBreakMode(0)
				.fontColor(Colors.secondaryContent)
				.fillWidth()
				.height(18) 
				.intrinsicSize(-1,-1)  
				.fontSize(13)      
				.keyPath("fileName"), 
			Spacer(), 
			Padding(15),
			PushButton("Replace", "Replace file", "Replace the sound file for this tile")   
				.toolTip("Replace the sound file for this tile")  
				.width(80)
				.keyPath("replaceFile"),
			Padding(12),
		]),		 
		
		VList([
			// Custom views that either appear or disappear based on if the controller section is visible
			View().height(6).showKey("showControllersSection"),
			View().height(11).hideKey("showControllersSection"),
		]),
		
		Divider().showKey("showControllersSection"), 
		
		ControllersSection().viewName("tileControllersSection"),
	])
	.makeView("GXSettingsView")
	.setAXTitle("Tile Inspector")
	.setAXHelp("Inspect and edit settings for “<gx>tile.title</gx>” tile")
	.viewName("tileInspectorView")
	.keyPath("");
} 

function LockReasonView() {
	return VStack([
		// Padding(7),
		HStack([
			Padding(15),
			View()
				.forePainter(ImagePainter("icon-warning-locked").tintColor(Colors.tileTint))
				.keyPath("")
				.size(24),
			FlexibleLabel()
				.keyPath("lockReason")
				.fontSize(13)
				.fontColor(Colors.tileTint)
				.fillFrame()
				.padding(10)
		])
			.makeView()
			.keyPath("")
			.fillFrame()
			.backPainter(Painter().radius(12).fill(MainThemeColor(Colors.tileTintPale, Colors.tileTintPale.withAlpha(0.3)))),
		Padding(2),
	]).makeView();
}

function MainContentView()
{  
	return VList([
		
		// Lock Reason view
		LockReasonView()
			.hPad(15) 
			.showKey("lockReason"),
			
		VStack([ 
			VStack([
				HStack([
					Padding(2.5),
					FramedTextField("Name", "title")
						.viewName("tileTitle")
						.enableKey("titleFieldEditable")
						.setAXTitle("Name")
						.fillWidth()
						.setAXHelp("Edit the name of the currently selected tiles"), 
					Padding(5),
					
					ZStack([ 
						AnimationView("star", Colors.tertiaryContent) 
							.fillFrame()
							.viewName("animationView1"), 
						AnimationView("star", Colors.tileTint)
							.keyPath("favorite")
							.showKey("favorite") 
							.fillFrame()
							.viewName("animationView3"),  
						AnimationView("star", Colors.tileTintLight.lightMode)
							.fillFrame()
							.viewName("animationView2"), // ++++  FIX ME - styling isn't right in dark mode 
						Button()
							.forePainter(Painter().fill(Colors.invisible.addStyle("pressed", Colors.mainBackground.withAlpha(0.2))))
							.backPainter(null)
							.checkBoxRole()  
							.toolTip("Favorite tile", "Unfavorite Tile") 
							.setAXTitle("Favorite Tile") 
							.fillFrame()
							.viewName("button")
					]) 	 
						.makeView("FGFaveButtonView")
						.keyPath("favorite")
						.disableKey("locked") 
						.size(26),
				
						
				]),
				Padding(4), 
				HStack([
					Padding(3),
					View("FGEmojiArtView") 
						.label("Emoji")
						.keyPath("tileIcon")  
						.height(29)
						.childViews([
							View("FGEmojiAddButton") 
								.shadow()
								.keyPath("tileIcon")
								.disableKey("artPickerDisabled")
								.toolTip("Add tile artwork") 
								.setAXTitle("Tile Artwork Selection")
								.setAXHelp("Add an emoji to display on this tile")
								.backPainter(CocoaButtonBack())
								.forePainter(ImagePainter("icon-emoji").tintColor(Colors.primaryContent)) 
								.viewName("addEmojiItem").width(40).fillHeight()
						]),
					Spacer(),  
					TabButton([
						View()
							.viewName("fillColor")
							.setValue(true,"gxIgnoreMouse")
							.disableKey("")
							.keyPath("")
							.backPainter(Painter()  
								.inset(4)
								.fill(Colors.tileTint.saturation(0.75), Colors.tileTint))
							.size(20), 
						View()
							.viewName("showPicker")
							.setValue(true,"gxIgnoreMouse")   
							.forePainter(ImagePainter("dropdown.arrow").tintColor(Colors.primaryContent))
							.size(20), 
					])
					.className("FGColorPicker") 
					.disableKey("locked")
					.toolTip("Set tile tint color") 
					.label("Color"),
				]),
			])
			.makeView() 
			.hPad(11), 
			
			Padding(4),

			HStack([  
				
				ZStack([
					 
					LayerView([   
						WaveLayer(false, Colors.invisible, Colors.tileTint, 6).viewName("backLayer"), 
						WaveLayer(true, Colors.tileTint, Colors.tileTintPale, 6).viewName("foreLayer"),
					])      
						.hPad(10)
						.viewName("waveLayerHost")
						.keyPath("")
						.addLayout("V:|-10-[self(>=90)]-0-|"), 
	
					// For AX Only
					ZStack([  
						Slider()
							.viewName("sliderTop")
							.knobPainter(Painter().size(4,2000).fill(Colors.editorPlayhead).stroke(1, Colors.mainBackground))
							.trackPainter(null)
							.animateKey("tile.player.playing")
							.keyPath("tile.player.trimmedPosition")
							.setAXTitle("Tile playback position")
							.fillFrame()
							.topPad(16), 
						Slider() 
							.viewName("sliderBottom")	
							.knobPainter(RoundKnob()).trackPainter(null)
							.topPad(0)
							.height(18)
							.keyPath("tile.player.trimmedPosition")
							.animateKey("tile.player.playing"),
					])
					.makeView()
					.showKey("playbackEnabled")
					.fillFrame(),
	
				])
					.makeView()
					.viewName("inspectorWaveView")
					.className("FGInspectorWaveView")
					.fillWidth()
					.fillHeight() 
					.toolTip("Set playback position for this tile") ,
				
				Padding(4), 
				LevelMeters()
					.fillHeight()
					.keyPath("tile.meter"),   
				Padding(0.5)
			])
			.makeView() 
			.hPad(15), 

			Padding(4),

			HStack([
				
				ToolbarButton("playback-back", "Back", "", "backBtn", [], null, Colors.tileTint)
					.enableKey("backBtn"),
			 
				Spacer(),
				
				AnimatedPlayButton().center(),
					
				Spacer(),
				
				ToolbarButton("playback-fade", "Fade Out", "", "fadeBtn", [], null, Colors.tileTint)
					.enableKey("fadeBtn")
					.toolTip("Fade tile")
					.setAXHelp("Fade out and stop the selected tile automatically"),
			])
			.makeView() 
			.hPad(8),
			
			Padding(2),

			HStack([
				Label("0.00.00") 
					.fontSize(10)
					.fontColor(Colors.secondaryContent)
					.animateKey("tile.playing")
					.keyPath("tile.currentTimeString"),
				Spacer(),
				PushButton("Edit", "Edit Audio Clip", null, shadow=false) 
					.keyPath("editFile")
					.toolTip("Open clip editor for the “<gx>tile.title</gx>” tile") 
					.width(100)
					.centerH(), 
				Spacer(),
				Label("0.00.00")
					.fontColor(Colors.secondaryContent)
					.fontSize(10)
					.animateKey("tile.playing")
					.keyPath("tile.remainingTimeString"),
			]).makeView().hPad(15), 
		]),

		Divider(),        

		TabButton([{ gxTitle: "Playback", 
					 gxValueKeyPath: "tile.playbackShowing", 
					 gxToolTip:"Switch to playback pane", 
					 axTitle: "Playback Pane",
					 axCheckboxRole: true,
					 axHelp: "Edit the output settings for the selected tile" }, 
				   { gxTitle: "Notes",	  
				   	 gxValueKeyPath: "tile.notesShowing", 	  
					 gxToolTip:"Switch to notes pane",    
					 axTitle: "Notes Pane",
					 axCheckboxRole: true,
					 axHelp: "Edit the notes and metadata for the selected tile" }]) 
					 .hPad(11),  
		
		
	])
	.rowSpacing(6)
	.topPad(9)
	.ignoreAX(false)
	.keyPath(""); 
}   
 
function PlaybackPane()
{
	return VStack([ 
		VStack([ 
			TileControls()
				.hPad(14),
			Padding(10),  
		])
			.makeView()
			.collapseKey("isListMode"),  
		PlayerVolumeView("Tile").hPad(15).height(40),
		Divider(),    
		Padding(4), 
        OutputDeviceSection().hPad(12),  
		Padding(4),
	])
	.makeView()
	.ignoreAX()
} 

function TileControls()
{
	return HStack([ 
		BigButton("control-loop", "Loop", "loop", "", onTip="Enable Looped Playback", offTip="Disable Looped Playback")
			.setAXTitle("Loop")
			.width(65)
			.setAXHelp("Automatically loop this tile")
			.disableKey("locked"),  
		View().viewName("pad1"),
		BigButton("control-solo-light", "Solo", "soloForMode", "", onTip="Enable Solo Mode",offTip="Disable Solo Mode") 
			.setAXTitle("Solo")
			.width(65)
			.setAXHelp("Stop other tiles when this tile is played")
			.disableKey("locked"),  
		View().viewName("pad2"),
		BigButton("control-resume", "Pausable", "resume", "", onTip="Enable Pausable",offTip="Disable Pausable")
			.setAXTitle("Pausable")
			.width(65)
			.setAXHelp("Pause this tile by default, rather than stop and reset to start")
			.disableKey("locked"),
		View().viewName("pad3"),
		BigButton("control-hold", "Hold", "holdToPlayForMode", "", onTip="Enable Hold to Play",offTip="Disable Hold to Play")
			.setAXTitle("Hold To Play")
			.setAXHelp("Only play this tile while it's hotkey is pressed")
			.width(65)
			.disableKey("locked")
			.wrapInFlexibleContainer(),  
	])
	.spaceViews(["pad1", "pad2", "pad3"])
	.makeView()   
}

function NotesPane(type)
{
	return ZStack([
		Divider(Colors.invisible),
		View()
			.backPainter(
				Painter()
					.fill(MainThemeColor(Colors.white, Colors.secondaryBackground))
					.stroke(1, Colors.secondaryBackgroundMix.addStyle("increasedContrast", Colors.contrastBorder))
					.radius(8)
			)
			.fillFrame(),
		ScrollView(TemplateView("TextView").intrinsicSize(-1,80).setValues({   
			gxResizeHeightToFitContent: true,
			gxEditable: true,
			gxValueKeyPath : "notes",
			drawsSelected: true,
			axTitle: `${type} Notes`,
			axHelp: `Set the notes for this ${type}`,   
			gxSelectionColor: Colors.tileTint.withAlpha(0.3), 
		}) 
			.enableKey("notesEnabled") 
			.fillWidth()  
		)  
		.backColor(MainThemeColor(Colors.white, Colors.secondaryBackground.darkMode))
		.vScroller(VScrollbarView(Colors.secondaryBackgroundMix, MainThemeColor(Colors.white, Colors.secondaryBackground)))
		.setValue(false, "gxAutohidesScrollers")
		.addLayout("H:|-2-[self]-2-|") // Insets account for the background being square, while the frame is rounded
		.addLayout("V:|-4-[self(>=95,<=220)]-4-|") 
		.hScroller(null), 
	])
	.makeView()
	.keyPath("")
}

function OutputDeviceSection()
{
    return View("FGOutputDeviceView").childViews([ 
		Label("Device") 
			.viewName("deviceLabel")
			.fontSize(13)
			.leftPad(2)
			.width(60),
		
		PopupButton()  
			.className("FGAudioDevicePopupButton")  
			.setAXTitle("Output device selector")
			.setAXHelp("Choose the output device for this tiles playback")
			.keyPath("outputDevice") 
			.menuItemsPath("deviceMenuItems")
			.disableKey("locked") 
			.viewName("outputPopup")
			.setValues({ 
				gxTitleKeyPath: "deviceTitle",
				gxDisableOnNil: false,
			}), 

		Label("Channels")
			.leftPad(2)
			.viewName("channelsLabel")
			.fontSize(13)
			.width(60),

		Label("L") 
			.viewName("leftLabel")
			.fontSize(13),

		Label("R") 
			.viewName("rightLabel")
			.fontSize(13),

		PopupButton() 
			.viewName("left") 
			.setAXTitle("Left channel selector")
			.setAXHelp("Choose the left output channel for this tiles playback") 
			.keyPath("outputChannelLeft")
			.disableKey("locked")
			.menuItemsPath("outputChannelMenuItems"),

		PopupButton() 
			.viewName("right")
			.setAXTitle("Right channel selector") 
			.setAXHelp("Choose the right output channel for this tiles playback") 
			.keyPath("outputChannelRight")
			.disableKey("locked")
			.menuItemsPath("outputChannelMenuItems"), 

	])  
	.intrinsicSize(-1,-1)
	.addLayout("V:|-2-[outputPopup]-2-[left]-2-|") 
	.addLayout("H:[deviceLabel]-(>=0)-[outputPopup]&alignCenterY") 
	.addLayout("H:[channelsLabel]-(>=0@200)-[left]-(>=0@200)-[right]&alignCenterY") 
	.addLayout("H:[leftLabel]-(>=0)-[left]&alignCenterY") 
	.addLayout("H:[rightLabel]-(>=0)-[right]&alignCenterY") 
	// .addLayout("H:|[self]|")  
};

function AnimatedPlayButton(pressedFill = Colors.mainBackground)
{
	function PlaybackButton()
	{
		return Button()  
			.ignoreAX()
			.hPad(0)
			.vPad(0)
			.backPainter(Painter().fill(Colors.invisible.addStyle("pressed", pressedFill.withAlpha(0.3))))
			.forePainter(null)   
			.keyPath("playBtn");
	};
	
	return ZStack([

		// For AX only
		Button() 
			.backPainter(null)
			.forePainter(null)
			.keyPath("pauseBtn")
			.toolTip("<gx>player.playButtonAXTitle</gx>")
			.fillFrame(),
		
		AnimationView("playback", Colors.tileTint.addStyle("playbackDisabled", Colors.secondaryContent.withAlpha(0.5))) 
			.fillFrame()
			.ignoreAX()
			.keyPath(""),
		
		PlaybackButton()
			.hideKey("resume")   
			.disableKey(null)   
			.keyPath("playBtn"),
		
		PlaybackButton()
			.showKey("resume")  
			.disableKey(null)   
			.keyPath("pauseBtn"),
			
	])
	.makeView()
	.size(40)
};
