markup.push
	({
		"EditorWindow": Window("Editor", VStack([

			Padding(Constants.headerHeight),

			Divider(),

			Padding(),

			ScrollView(
				TimeLineView() 
					.menuItems([
						MenuItem("Remove Selection").action("delete:"),
						MenuItem("Crop Selection").action("crop:"),
						MenuItem("-"),
						MenuItem("Fade Selection In").action("fadeInSelection:"),
						MenuItem("Fade Selection Out").action("fadeOutSelection:"),
					]).viewName("timeLineEditor"),
			)
				.fillFrame()
				.hPad(0)
				.vScroller(null)
				.setValue(true, "gxOverlaysScroller")
				.setValues({
					gxAutohidesScrollers: true,
					nsClass: "Farrago.FGTimeLineScrollView",
				}).setAXRole("Clip Timeline"),

			// Transport Section 

			ZStack([

				HStack([
					VolumeIndicator(MainThemeColor(Colors.black, Colors.white))
						.toolTip("Un-mute", "Mute")
						.keyPath("volumeIndicator")
						.stylesKeyPath("volumeIndicatorStyles"),
					Padding(5),
					VolumeSlider()
						.className("Farrago.FGNoFocusSlider")
						.trackPainter(SliderTrack(Colors.fontColor))
						.setAXTitle("Editor Volume")
						.setAXFormatter(PercentageFormatter())
						.width(150)
						.keyPath("fullVolume"),

				]).makeView().centerV().leftPad(5),

				HStack([
					Label()
						.keyPath("player.currentTimeFormatted")
						.animateKey("player.playing")
						.fontSize(13)
						.fontColor(Colors.fontColor)
						.textAlign(1,0.5)
						.width(80) 
						,

					Padding(20),

					Button()
						.toolTip("Skip backward 5 seconds")
						.backPainter(ImagePainter("control-skip-backwards").tintColor(Colors.tileTint.showPressed()))
						.size(40)
						.keyPath("rewind"),

					Button()
						.toolTip("Return to clip start")
						.backPainter(ImagePainter("control-back").tintColor(Colors.tileTint.showPressed()))
						.size(40)
						.keyPath("backBtn"),

					Padding(),

					AnimatedPlayButton(Colors.mainBackground)
						.size(40),

					Padding(),

					Button()
						.toolTip("Skip forward 5 seconds").backPainter(ImagePainter("control-skip-forward").tintColor(Colors.tileTint.showPressed()))
						.keyPath("fastForward")
						.size(40),

					Padding(20),

					Label()
						.keyPath("player.remainingTimeFormatted")
						.animateKey("player.playing")
						.width(100)
						.fontColor(Colors.fontColor)
						.fontSize(13),
						,

				])
					.makeView()
					.height(50)
					.centerH(),

			])
				.makeView()
				.hPad(10),
			Divider(),
			Padding(11),
			HStack([
				Spacer(),
				PushButton("Save", "Save Clip", "Commit all edits for this clip")
					.toolTip("Save clip edits") 
					.keyPath("saveButton")
					.width(120)
					.keyEquivalent("cmd_s"),
				Padding(12),
			])
				.makeView("GXSettingsView")
				.viewName("footerView"),
			Padding(11),
			// Create invisible button to handle returnToZero shortcut
			Button("").keyEquivalent("\r").size(0).setValue("returnToZero:", "gxAction"),

		])
			.background(Color(Colors.mainBackground))
			.addLayout("H:[self(>=950)]") // Big enough to fit "Editing: Car Interior Ambience"
			.addLayout("V:[self(>=500)]"))
			.setValues({
				"gxInitialWindowSize": "{800,500}",
				toolbarView: EditorToolbarView(),
			})
			.className("FGCustomTitleBarWindow"),
	})

function EditorToolbarView() {

	return HStack([

		Label()
			.lineBreakMode(0)
			.fillWidth(200)
			.height(18)
			.textAlign(0, 1)
			.intrinsicSize(-1, -1)
			.addLayout("H:[self(>=40)]")
			.fontSize(14)
			.fontColor(Colors.primary)
			.medium()
			.keyPath("editorTitle"),

		ToolbarButtonDark("zoomOut", "Zoom Out", "", "zoomOut")
			.ignoreAX(),
		ToolbarButtonDark("zoomIn", "Zoom In", "", "zoomIn")
			.ignoreAX(),

		Padding(20),

		ToolbarButtonDark("cut", "Remove Selection", "", "deleteSelectedRange")
			.disableKey(null)
			.setAXHelp("Remove selected audio"),

		ToolbarButtonDark("crop", "Crop Selection", "", "cropSelection")
			.disableKey(null)
			.setAXHelp("Crop audio to selection"),

		ToolbarButtonDark("control-fade-in", "Fade In", "", "fadeIn")
			.setAXHelp("Fade in the audio of any selected portions of selected clips")
			.disableKey(null),
		ToolbarButtonDark("control-fade-out", "Fade Out", "", "fadeOut")
			.disableKey(null)
			.setAXHelp("Fade out the audio of any selected portions of selected clips"),

		Padding(20),

		ToolbarButtonDark("control-undo", "Undo", "", "undo")
			.disableKey(null)
			.setAXHelp("Undo last change made"),

		ToolbarButtonDark("control-redo", "Redo", "", "redo")
			.disableKey(null)
			.setAXHelp("Redo last change made"),

		ToolbarButtonDark("control-reset", "Revert to Original", "", "revert")
			.disableKey(null)
			.setAXHelp("Revert all changes made to all clips in the editor"),

		LevelMeters().height(40).viewName("meters").keyPath("meterNode"),

		View(),

	], 2)
		.makeView("GXSettingsView")
		.addLayout(`V:[self(>=${Constants.headerHeight})]`)
		.viewName("toolbarView")
		.keyPath("");
}

function TimeLineView() {
	return View("Farrago.FGTimelineEditorView")
		.childViews([
			View("Farrago.FGTimeView")
				.keyPath("")
				.setValues({
					"gxDetentPainter": Painter().fill(Colors.tertiary),
					"gxLabelPainter": LabelPainter("").setValue(2, "gxLineBreakMode").fontSize(9).color(Colors.secondaryContent).offset(0, 8).align(0, 1), 
					"gxIgnoreScrollWheel": true,
				}).viewName("timeLineView")
				.forePainter(Painter().fill(Colors.mainBackground)),

			Slider()
				.className("Farrago.AXSliderView")
				.endGap(Constants.timelineInset)
				.fillWidth()
				.knobPainter(Painter().size(4, 10000).fill(Colors.editorPlayhead).stroke(1, Colors.mainBackground))
				.trackPainter(null)
				.viewName("playhead")
				.keyPath("playheadPosition")
				.animateKey("player.playing") 
				.setAXTitle("Playback Position")
				.setValues({
					"gxIgnoreScrollWheel": true,
					"gxIgnoreTrackClicks": true, 
					"gxKeystrokeStepSize": 0.1,
					"axFormatter": TimeFormatter(true),
				}),

			ClipView()
				.viewName("clipView")
				.addLayout("V:|-40-[self]-5-|"),


			Slider()
				.endGap(Constants.timelineInset)
				.fillWidth()
				.trackPainter(null).setValues({
					gxIgnoreScrollWheel: true,
					// "gxIgnoreTrackClicks": true,
				})
				.height(20)
				.topPad(8)
				.animateKey("player.playing")
				
				.viewName("playheadKnob")
				.keyPath("player.currentPosition")
		])
}

function FadeSlider(fadeIn) {
	return Slider().height(16).trackPainter(null)
		.endGap(30)
		.knobPainter(Painter().size(60, 16).fill(Colors.tileTint).radius(4))
		.knobPainterExtra(LabelPainter(fadeIn ? "Fade In" : "Fade Out").color(Colors.white))
		.setValue(true, "gxIgnoreTrackClicks")
		.setValue(true, "gxIgnoreScrollWheel")
} 